// Copyright 2006 all rights reserved; see LICENSE file for BSD-style license

package edu.berkeley.sbp;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.Parser.Table.*;
import edu.berkeley.sbp.Sequence.Position;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

final class Result implements GraphViz.ToGraphViz {

    private Forest f;
    private Node parent;
    private GSS.Phase phase;
    private Position reduction;
    private HashSet<Node> children = new HashSet<Node>();
    private boolean destroyed = false;

    public Position reduction() { return reduction; }
    public GSS.Phase phase() { return phase; }
    public Forest getForest() { return f; }
    public Node parent() { return parent; }
    public void addChild(Node child) { children.add(child); }
    public void removeChild(Node child) { children.remove(child); }

    public boolean usedByAnyNode() { return children.size() > 0; }
    public boolean usedByNonDoomedNode() {
        for(Node n : children)
            if (!n.state().doomed)
                return true;
        return false;
    }

    public String toString() { return super.toString()+"->"+parent(); }

    public void check() { if (children.size()==0) destroy(); }
    public void destroy() {
        if (destroyed) return;
        if (parent==null) return;  // never destroy the "primordeal" result
        destroyed = true;
        if (parent() != null) {
            parent().removeChild(this);
            parent().check();
        }
        OUTER: while(true) {
            for(Node n : children) {
                children.remove(n);
                n.removeResult(this);
                n.check();
                continue OUTER;
            }
            break;
        }
    }

    public Result(Forest f, Node parent, Position reduction) {
        this.f = f;
        this.reduction = reduction;
        this.parent = parent;
        if (parent != null) parent.addChild(this);
        if (parent != null) phase = parent.phase();
    }

    // GraphViz //////////////////////////////////////////////////////////////////////////////

    public GraphViz.Node toGraphViz(GraphViz gv) {
        if (gv.hasNode(this)) return gv.createNode(this);
        GraphViz.Node n = gv.createNode(this);
        n.label = ""+f;
        n.shape = "rectangle";
        if (parent()!=null) n.edge(parent, "");
        n.color = "blue";
        if (phase() != null)
            ((GraphViz.Group)phase().toGraphViz(gv)).add(n);
        return n;
    }
    public boolean isTransparent() { return false; }
    public boolean isHidden() { return false; }

}